@@ -1,6 +1,6 @@
 
 /*
- * $Header: /cvs/gnome/gperfmeter/src/props.c,v 1.37 2004/11/15 17:12:49 richb Exp $
+ * $Header: /cvs/gnome/gperfmeter/src/props.c,v 1.45 2005/03/10 18:38:17 richb Exp $
  *
  * Copyright (c) 1990-2004 Sun Microsystems, Inc. All Rights Reserved.
  *
@@ -25,6 +25,7 @@
 #include <ctype.h>
 #include <math.h>
 #include <glade/glade.h>
+#include <glib/gstrfuncs.h>
 
 #include "callbacks.h"
 #include "perfmeter.h"
@@ -69,7 +70,7 @@ enum {
 
    /* New Box Controls */
    P_TVIEW, P_TPREF, P_TCONTROL, P_TABSEP, P_SHOWBOX, 
-   P_SAMPLEMOD, P_SAMPLEMENU,
+   P_SAMPLEMOD,
    P_DIR , P_LMULT , P_CHKBOX, P_TIMEBOX, P_LSAMPLE, P_LOGBOX , P_OUTBOX,  
    P_RSHOW, P_NOTE, P_BBOX,
 
@@ -78,13 +79,12 @@ enum {
    P_PKTSTH, P_COLLTH, P_ERRSTH,
 
    /* Miscellaneous controls */
-   P_VERTICAL, P_HORIZONTAL, P_LINE, P_SOLID, P_LOCAL, P_REMOTE, P_HOSTS,
-   P_HOSTSCOMBO,
+   P_VERTICAL, P_HORIZONTAL, P_LINE, P_SOLID, P_LOCAL, P_REMOTE, P_HOSTSCOMBO,
    P_TIME, P_LOG, P_LOGFILE, P_LIVE, P_STRIP, P_BOTH, P_PREFSEP,P_PREFSEP2,
    P_TVIEWBOT,P_TPREFBOT, P_MCTL,
 
 
-   P_COLLECT, P_MENUBAR, P_RESIZE, P_STATUS, 
+   P_COLLECT, P_HOSTNAME, P_MENUBAR, P_RESIZE, P_STATUS, 
    P_TITLE, P_CURMAX, P_MAXMAX, P_MINMAX, P_LIMIT, P_RANGEINC, P_METERCOLOR, 
    P_CANVASCOLOR, 
    P_LIMITCOLOR,
@@ -124,10 +124,10 @@ static gboolean IsValueChanged();
 static gboolean PmColorChk(GdkColor *PMColor, GdkColor *UIPColor);
 
 static void ApplyProps();
-static void colorpicker_set_default_colors(GnomeColorPicker *, GdkColor, gint);
+static void colorpicker_set_default_colors(GtkColorButton *, GdkColor, gint);
 static void CreateProps();
 static void InstantApply();
-static void PMColorSetCB(GtkWidget *, guint, guint, guint, guint, gint);
+static void PMColorSetCB(GtkColorButton *picker, gpointer user_data);
 static void PropsSpinValueChanged(GtkSpinButton *, gpointer);
 static void ResetSheet(void);
 static void SetSensitive(GtkWidget *, gboolean);
@@ -163,6 +163,7 @@ typedef struct _PmUIProps {
     int SampleModifier;
     gboolean Log;
     gboolean ShowLimit;
+    gboolean ShowHostName;
     gboolean ShowMenubar;
     gboolean ShowStripChart;
     gboolean ShowLiveActionBar;
@@ -197,10 +198,10 @@ static void
 SetPickerColor(int ci, int mi, gboolean is_meter)
 {
     if (is_meter) {
-        colorpicker_set_default_colors(GNOME_COLOR_PICKER(UIP->Controls[ci]), 
+        colorpicker_set_default_colors(GTK_COLOR_BUTTON(UIP->Controls[ci]), 
                                        PM->Meters[mi].MeterColor, mi);
     } else {
-        colorpicker_set_default_colors(GNOME_COLOR_PICKER(UIP->Controls[ci]), 
+        colorpicker_set_default_colors(GTK_COLOR_BUTTON(UIP->Controls[ci]), 
                                        PM->Meters[mi].LimitColor, mi);
     }
 }
@@ -217,9 +218,9 @@ SetPropsToggle(int ci, gboolean state)
 
 /*ARGSUSED*/
 static void
-SampleMenuCB(GtkMenuItem *mi, gpointer user_data)
+SampleMenuCB(GtkComboBox *combo_box, gpointer user_data)
 {
-    int value = (int) g_object_get_data(G_OBJECT(mi), "SampleValue");
+    int value = gtk_combo_box_get_active(combo_box);
 
     if (value == 0) {
         UIP->SampleModifier = 1;
@@ -233,20 +234,6 @@ SampleMenuCB(GtkMenuItem *mi, gpointer user_data)
 }
 
 
-static void
-PmAddSampleValue(char *name, int value)
-{
-    GtkWidget *menu_item = gtk_menu_item_new_with_label(name);
-
-    g_object_set_data(G_OBJECT(menu_item), "SampleValue", (gpointer) value);
-    g_signal_connect(G_OBJECT(menu_item), "activate",
-                     G_CALLBACK(SampleMenuCB), NULL);
-    gtk_menu_shell_append(GTK_MENU_SHELL(UIP->Controls[P_SAMPLEMENU]), 
-                          menu_item);
-    gtk_widget_show_all(UIP->Controls[P_SAMPLEMENU]);
-}
-
-
 static void
 SetupColorPickerTooltips()
 {
@@ -317,6 +304,24 @@ SetupColorPickerTooltips()
 }
 
 
+static void
+set_remote_host_list(GtkWidget *combo_box, GList *list)
+{
+    gint i, len;
+    gchar *str;
+
+    if (list != NULL) {
+        len = g_list_length(list);
+
+        for (i = 0; i < len; i++) {
+            str = g_strdup((gchar *) g_list_nth_data(list, i));
+
+            gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box),  str);
+        }
+    }
+}
+
+
 /*ARGSUSED*/
 static gboolean
 CommandKeyEvent(GtkWidget *widget, GdkEventKey *event, gpointer data)
@@ -339,7 +344,7 @@ CreateProps()
 {
     int i;
 
-    UIP = (PmUIProps *) calloc(1, sizeof(PmUIProps));
+    UIP = (PmUIProps *) g_malloc0(sizeof(PmUIProps));
     
     /* Loads the preferences dialog from the Glade file */
     xml = glade_xml_new(GPERF_GLADEDIR "/gperfmeter.glade", NULL, NULL);
@@ -365,17 +370,14 @@ CreateProps()
     UIP->Controls[P_COLL] = SetPropsData("Collisions_CheckB", P_COLL);
     UIP->Controls[P_ERRS] = SetPropsData("Errors_CheckB",     P_ERRS);
 
-    UIP->Controls[P_HOSTS]      = glade_xml_get_widget(xml, "Hosts_Entry");
     UIP->Controls[P_HOSTSCOMBO] = glade_xml_get_widget(xml, "Hosts_Combo");
-    if (remote_hosts != NULL) {
-        gtk_combo_set_popdown_strings(GTK_COMBO(UIP->Controls[P_HOSTSCOMBO]),
-                                      remote_hosts);
-    }
+    set_remote_host_list(UIP->Controls[P_HOSTSCOMBO], remote_hosts);
 
     SetSensitive(UIP->Controls[P_HOSTSCOMBO], PM->IsRemote);
 
-    g_signal_connect(G_OBJECT(UIP->Controls[P_HOSTS]), "key_press_event",
-                     GTK_SIGNAL_FUNC(EntryKeyHandler), NULL);
+    g_signal_connect(
+        G_OBJECT(GTK_ENTRY(GTK_BIN(UIP->Controls[P_HOSTSCOMBO])->child)),
+        "key_press_event", GTK_SIGNAL_FUNC(EntryKeyHandler), NULL);
 
     UIP->Controls[P_LOG]       = glade_xml_get_widget(xml, "LogSamples");
     UIP->Controls[P_LOGFILE]   = glade_xml_get_widget(xml, "LogFile_Entry");
@@ -453,13 +455,9 @@ CreateProps()
     }
 
     UIP->Controls[P_SAMPLEMOD]  = SetPropsData("SampleMod", P_SAMPLEMOD);
-    UIP->Controls[P_SAMPLEMENU] = gtk_menu_new();
-    PmAddSampleValue(_("Seconds"), 0);
-    PmAddSampleValue(_("Minutes"), 1);
-    PmAddSampleValue(_("Hours"),   2);
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(UIP->Controls[P_SAMPLEMOD]),
-                             UIP->Controls[P_SAMPLEMENU]);
-    gtk_option_menu_set_history(GTK_OPTION_MENU(UIP->Controls[P_SAMPLEMOD]), 1);
+    gtk_combo_box_set_active(GTK_COMBO_BOX(UIP->Controls[P_SAMPLEMOD]), 1);
+    g_signal_connect(G_OBJECT(UIP->Controls[P_SAMPLEMOD]), "changed",
+                     G_CALLBACK(SampleMenuCB), NULL);
     UIP->SampleModifier = 1;
 
     UIP->Controls[P_HORIZONTAL] = SetPropsData("HorizontalRB",  P_HORIZONTAL);
@@ -472,6 +470,7 @@ CreateProps()
     UIP->Controls[P_LOCAL]      = SetPropsData("LocalRB",       P_LOCAL);
     UIP->Controls[P_REMOTE]     = SetPropsData("RemoteRB",      P_REMOTE);
     UIP->Controls[P_LOG]        = SetPropsData("LogSamples",    P_LOG);
+    UIP->Controls[P_HOSTNAME]   = SetPropsData("ShowHostName",  P_HOSTNAME);
     UIP->Controls[P_MENUBAR]    = SetPropsData("HideMenuBar",   P_MENUBAR);
     UIP->Controls[P_SHOWLIMIT]  = SetPropsData("ShowLimit",     P_SHOWLIMIT);
     UIP->Controls[P_TIME]       = SetPropsData("TimeSB",        P_TIME);
@@ -528,6 +527,7 @@ IsValueChanged()
         (UIP->ShowStripChart    != PM->ShowStripChart) ||
         (UIP->Log               != PM->Log) ||
         (UIP->ShowLimit         != PM->ShowLimit) ||
+        (UIP->ShowHostName      != PM->ShowHostName) || 
         (UIP->ShowMenubar       != PM->ShowMenubar) || 
         (UIP->IsRemote          != PM->IsRemote)) {
         return(TRUE);
@@ -612,6 +612,10 @@ PropsToggleCB(GtkToggleButton *tb, gpointer user_data)
             SetSensitive(UIP->Controls[P_LOGFILE], set);
             break;
 
+        case P_HOSTNAME:
+            UIP->ShowHostName = set;
+            break;
+
         case P_MENUBAR:
             UIP->ShowMenubar = !set;
             break;
@@ -692,11 +696,11 @@ PropsMeterCB(GtkToggleButton *tb, gpointer user_data)
     }
 
 #ifdef DEBUG
-    PRINTF("Display List = %d %d %d %d %d %d %d %d %d %d\n",
-           UIP->DisplayList[0], UIP->DisplayList[1], UIP->DisplayList[2],
-           UIP->DisplayList[3], UIP->DisplayList[4], UIP->DisplayList[5],
-           UIP->DisplayList[6], UIP->DisplayList[7], UIP->DisplayList[8],
-           UIP->DisplayList[9]);
+    g_print("Display List = %d %d %d %d %d %d %d %d %d %d\n",
+            UIP->DisplayList[0], UIP->DisplayList[1], UIP->DisplayList[2],
+            UIP->DisplayList[3], UIP->DisplayList[4], UIP->DisplayList[5],
+            UIP->DisplayList[6], UIP->DisplayList[7], UIP->DisplayList[8],
+            UIP->DisplayList[9]);
 #endif
 }
 
@@ -728,7 +732,7 @@ PropsLimitCB(GtkEditable *widget, gpointer user_data)
     char *s;
     int limit;
     int code = (int) g_object_get_data(G_OBJECT(widget), "PropsData");
-    char buf[20];
+    gchar *buf;
 
     if ((s = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget))))) {
         value = strtol(s, NULL, 10);
@@ -751,18 +755,20 @@ PropsLimitCB(GtkEditable *widget, gpointer user_data)
         gtk_widget_show_all(dialog);
 
         value = UIP->Meters[code].Limit;
-        SPRINTF(buf, "%ld", value);
+        buf = g_strdup_printf("%ld", value);
         gtk_entry_set_text(GTK_ENTRY(widget), buf);
+	g_free(buf);
     } else if (value < 0) {
         value = 0;
-        SPRINTF(buf, "%ld", value);
+        buf = g_strdup_printf(buf, "%ld", value);
         gtk_entry_set_text(GTK_ENTRY(widget), buf);
+	g_free(buf);
     }
 
     limit = (int) value;
 
 #ifdef DEBUG
-    PRINTF("Limit %d Value = %d\n", code, limit);
+    g_print("Limit %d Value = %d\n", code, limit);
 #endif
 
     UIP->Meters[code].Limit = (int) PmMax(limit, 0);
@@ -771,18 +777,19 @@ PropsLimitCB(GtkEditable *widget, gpointer user_data)
 
 /*ARGSUSED*/
 static void 
-PropsSpinValueChanged (GtkSpinButton *widget, gpointer data)
+PropsSpinValueChanged(GtkSpinButton *widget, gpointer data)
 {
-    char buf[20];
-    int limit;
+    gint limit;
+    gchar *buf;
 
     limit = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
 
     if (limit > PmMAX_SAMPLETIME) {
         limit = PmMAX_SAMPLETIME;
-        SPRINTF(buf, "%d", limit);
+        buf = g_strdup_printf("%d", limit);
         gtk_spin_button_set_value(GTK_SPIN_BUTTON(UIP->Controls[P_TIME]),
                                   (gfloat) limit);
+	g_free(buf);
     }
 
     UIP->SampleInterval = limit;
@@ -839,8 +846,8 @@ PmHideProps()
 void 
 PmResetProps()
 {
-    char str[20];
-    int i;
+    gchar *str = NULL;
+    gint i;
 
     if (!UIP || !UIP->Showing) {
         return;
@@ -859,6 +866,7 @@ PmResetProps()
     UIP->SampleInterval      = PM->SampleInterval;
     UIP->Log                 = PM->Log;
     UIP->ShowLimit           = PM->ShowLimit;
+    UIP->ShowHostName        = PM->ShowHostName;
     UIP->ShowMenubar         = PM->ShowMenubar;
     UIP->ShowStripChart      = PM->ShowStripChart; /* S C */
     UIP->ShowLiveActionBar   = PM->ShowLiveActionBar;
@@ -911,9 +919,11 @@ PmResetProps()
         SetPropsToggle(P_LOCAL, TRUE);
     }
 
-    SPRINTF(str, "%d", UIP->SampleInterval);
+    str = g_strdup_printf("%d", UIP->SampleInterval);
     gtk_entry_set_text(GTK_ENTRY(UIP->Controls[P_TIME]), str);
+    g_free(str);
 
+    SetPropsToggle(P_HOSTNAME, UIP->ShowHostName);
     SetPropsToggle(P_MENUBAR, !UIP->ShowMenubar);
     SetPropsToggle(P_LOG, UIP->Log);
     SetPropsToggle(P_SHOWLIMIT, UIP->ShowLimit);
@@ -921,16 +931,13 @@ PmResetProps()
 
     /* Set the selected sample modifier UI */
     if (PM->SampleModifier == 1) {
-        gtk_option_menu_set_history(
-                            GTK_OPTION_MENU(UIP->Controls[P_SAMPLEMOD]), 0);
+        gtk_combo_box_set_active(GTK_COMBO_BOX(UIP->Controls[P_SAMPLEMOD]), 0);
     }
     if (PM->SampleModifier == 60) {
-        gtk_option_menu_set_history(
-                            GTK_OPTION_MENU(UIP->Controls[P_SAMPLEMOD]), 1);
+        gtk_combo_box_set_active(GTK_COMBO_BOX(UIP->Controls[P_SAMPLEMOD]), 1);
     }
     if (PM->SampleModifier == 3600) {
-        gtk_option_menu_set_history(
-                            GTK_OPTION_MENU(UIP->Controls[P_SAMPLEMOD]), 2);
+        gtk_combo_box_set_active(GTK_COMBO_BOX(UIP->Controls[P_SAMPLEMOD]), 2);
     }
 
     ResetSheet();
@@ -990,13 +997,14 @@ PmResetCurMax(int meter)
 static void
 SetLimit(int ci, int mi)
 {
-    char buf[100];
+    gchar *buf;
 
-    SPRINTF(buf, "%d", UIP->Meters[mi].Limit);
+    buf = g_strdup_printf("%d", UIP->Meters[mi].Limit);
     gtk_entry_set_text(GTK_ENTRY(UIP->Controls[ci]), buf);
+    g_free(buf);
 
 #ifdef DEBUG
-    PRINTF("\tLimit = %d\n", UIP->Meters[mi].Limit);
+    g_print("\tLimit = %d\n", UIP->Meters[mi].Limit);
 #endif
 }
 
@@ -1007,7 +1015,7 @@ static void
 ResetSheet()
 {
 #ifdef DEBUG
-    PRINTF("Reset Meters\n");
+    g_print("Reset Meters\n");
 #endif
 
     SetLimit(P_CPUTH, P_CPU);
@@ -1090,7 +1098,7 @@ ApplyProps()
     PM->Log = UIP->Log;
     PM->SampleModifier = UIP->SampleModifier;
     s = g_strdup(gtk_entry_get_text(GTK_ENTRY(UIP->Controls[P_LOGFILE])));
-    PmExpandPathname(s, PM->LogFilename);
+    PM->LogFilename = expand_tilde(s);
     g_free(s);
 
     if (PM->Log) {
@@ -1098,10 +1106,12 @@ ApplyProps()
             gtk_entry_set_text(GTK_ENTRY(UIP->Controls[P_LOGFILE]), 
                                PM->LogFilename);
         } else {                              /* Couldn't open log file */
-            char buf[512];
+            gchar *buf;
 
-            SPRINTF(buf, "Can't open %s", PM->LogFilename);
+            buf = g_strdup_printf("Can't open %s", PM->LogFilename);
             gtk_entry_set_text(GTK_ENTRY(UIP->Controls[P_LOGFILE]), buf);
+            g_free(buf);
+
             gdk_flush();
             SLEEP(2);
             gtk_entry_set_text(GTK_ENTRY(UIP->Controls[P_LOGFILE]), 
@@ -1127,9 +1137,10 @@ ApplyProps()
         /* The remote flag has changed */
 
         if (PM->IsRemote) {
-            s = g_strdup(gtk_entry_get_text(GTK_ENTRY(UIP->Controls[P_HOSTS])));
+            s = g_strdup(gtk_entry_get_text(GTK_ENTRY(
+                                GTK_BIN(UIP->Controls[P_HOSTSCOMBO])->child)));
             if (s != NULL &&
-                strlen(s) && (strcasecmp(s, PM->LocalMachine) != 0)) {
+                strlen(s) && (g_ascii_strcasecmp(s, PM->LocalMachine) != 0)) {
                 PmStrcpy(PM->Hostname, s);
 
                 if (PmMonitor()) {
@@ -1179,9 +1190,11 @@ ApplyProps()
         }
     } else if (PM->IsRemote) {
         /* Check to see if the machine has changed */
-        s = g_strdup(gtk_entry_get_text(GTK_ENTRY(UIP->Controls[P_HOSTS])));
-        if (s != NULL && strlen(s) && (strcasecmp(s, PM->LocalMachine) != 0)) {
-            if (strcasecmp(PM->Hostname, s)) {
+        s = g_strdup(gtk_entry_get_text(GTK_ENTRY(
+                                GTK_BIN(UIP->Controls[P_HOSTSCOMBO])->child)));
+        if (s != NULL && strlen(s) && 
+            g_ascii_strcasecmp(s, PM->LocalMachine) != 0) {
+            if (g_ascii_strcasecmp(PM->Hostname, s)) {
                 PmStrcpy(PM->Hostname, s);
                 PMMONITOR();
                 redisplay = TRUE;
@@ -1198,6 +1211,14 @@ ApplyProps()
         g_free(s);
     }          
 
+    if (PmAssign(PM->ShowHostName, UIP->ShowHostName)) {
+        if (PM->ShowHostName) {
+            gtk_widget_show(UI->HostName);
+        } else {
+            gtk_widget_hide(UI->HostName);
+        }
+    }
+
     if (PmAssign(PM->ShowMenubar, UIP->ShowMenubar)) {
         PmSetMinSize();
         if (PM->ShowMenubar) {
@@ -1240,15 +1261,13 @@ ApplyProps()
 
 /*ARGSUSED*/
 static void
-PMColorSetCB(GtkWidget *picker, guint r, guint g, guint b, guint a, gint index)
+PMColorSetCB(GtkColorButton *picker, gpointer user_data)
 {
     gint data;    
     GdkColor color;
+    int index = (int) user_data;
 
-    color.red = r;
-    color.green = g;
-    color.blue = b;
-
+    gtk_color_button_get_color(picker, &color);
     data = (gint) g_object_get_data(G_OBJECT(picker), "PropsData");
 
     switch (index) {
@@ -1342,6 +1361,10 @@ PMColorSetCB(GtkWidget *picker, guint r, guint g, guint b, guint a, gint index)
 static void 
 SetSensitive(GtkWidget *w, gboolean sensitive)
 {
+    if (w == NULL) {
+        return;
+    }
+
     if (sensitive) {
         if (!GTK_WIDGET_IS_SENSITIVE(w)) {
             gtk_widget_set_sensitive(w, TRUE);
@@ -1380,8 +1403,7 @@ PmAddHostToList(char *name)
     }
 
     if (UIP != NULL) {
-        gtk_combo_set_popdown_strings(GTK_COMBO(UIP->Controls[P_HOSTSCOMBO]),
-                                      remote_hosts);
+        set_remote_host_list(UIP->Controls[P_HOSTSCOMBO], remote_hosts);
     }
 }
 
@@ -1453,12 +1475,11 @@ DismissPropsCB(GtkWidget *widget, GdkEvent *event, gpointer user_data)
 
 
 static void
-colorpicker_set_default_colors(GnomeColorPicker *gnome_colorpicker, 
+colorpicker_set_default_colors(GtkColorButton *color_picker, 
                                GdkColor color, gint index)
 {
-    gnome_color_picker_set_i16(GNOME_COLOR_PICKER(gnome_colorpicker),
-                               color.red, color.green, color.blue, 0xffff);
-    g_signal_connect(G_OBJECT(gnome_colorpicker), "color_set",
+    gtk_color_button_set_color(GTK_COLOR_BUTTON(color_picker), &color);
+    g_signal_connect(G_OBJECT(color_picker), "color-set",
              G_CALLBACK(PMColorSetCB), (gpointer) index);    
 }
 
